FROM_ICAO_PREFIX = {
    'A1': 'Antarctica (United States)',
    'AG': 'Solomon Islands',
    'AN': 'Nauru',
    'AY': 'Papua New Guinea',
    'BG': 'Greenland',
    'BI': 'Iceland',
    'BK': 'Kosovo',
    'C0': 'Canada',
    'C1': 'Canada',
    'C2': 'Canada',
    'C3': 'Canada',
    'C4': 'Canada',
    'C5': 'Canada',
    'C6': 'Canada',
    'C7': 'Canada',
    'C8': 'Canada',
    'C9': 'Canada',
    'CA': 'Canada',
    'CB': 'Canada',
    'CC': 'Canada',
    'CD': 'Canada',
    'CE': 'Canada',
    'CF': 'Canada',
    'CG': 'Canada',
    'CH': 'Canada',
    'CJ': 'Canada',
    'CK': 'Canada',
    'CL': 'Canada',
    'CM': 'Canada',
    'CN': 'Canada',
    'CO': 'Canada',
    'CP': 'Canada',
    'CR': 'Canada',
    'CS': 'Canada',
    'CT': 'Canada',
    'CV': 'Canada',
    'CW': 'Canada',
    'CX': 'Canada',
    'CY': 'Canada',
    'CZ': 'Canada',
    'DA': 'Algeria',
    'DB': 'Benin',
    'DF': 'Burkina Faso',
    'DG': 'Ghana',
    'DI': "Cote d'Ivoire",
    'DN': 'Nigeria',
    'DR': 'Niger',
    'DT': 'Tunisia',
    'DX': 'Togolese Republic',
    'EB': 'Belgium',
    'ED': 'Germany (civil)',
    'EE': 'Estonia',
    'EF': 'Finland',
    'EG': 'United Kingdom (and Crown dependencies)',
    'EH': 'Netherlands',
    'EI': 'Ireland',
    'EK': 'Denmark and the Faroe Islands',
    'EL': 'Luxembourg',
    'EN': 'Norway',
    'EP': 'Poland',
    'ES': 'Sweden',
    'ET': 'Germany (military)',
    'EV': 'Latvia',
    'EY': 'Lithuania',
    'FA': 'South Africa',
    'FB': 'Botswana',
    'FC': 'Republic of the Congo',
    'FD': 'Eswatini',
    'FE': 'Central African Republic',
    'FG': 'Equatorial Guinea',
    'FH': 'Saint Helena, Ascension and Tristan da Cunha',
    'FI': 'Mauritius',
    'FJ': 'British Indian Ocean Territory',
    'FK': 'Cameroon',
    'FL': 'Zambia',
    'FM': 'Comoros, France (Mayotte and Reunion), and Madagascar',
    'FN': 'Angola',
    'FO': 'Gabon',
    'FP': 'Sao Tome and Príncipe',
    'FQ': 'Mozambique',
    'FS': 'Seychelles',
    'FT': 'Chad',
    'FV': 'Zimbabwe',
    'FW': 'Malawi',
    'FX': 'Lesotho',
    'FY': 'Namibia',
    'FZ': 'Democratic Republic of the Congo',
    'GA': 'Mali',
    'GB': 'The Gambia',
    'GC': 'Spain (Canary Islands)',
    'GE': 'Spain (Ceuta and Melilla)',
    'GF': 'Sierra Leone',
    'GG': 'Guinea-Bissau',
    'GL': 'Liberia',
    'GM': 'Morocco',
    'GO': 'Senegal',
    'GQ': 'Mauritania',
    'GS': 'Western Sahara',
    'GU': 'Guinea',
    'GV': 'Cape Verde',
    'HA': 'Ethiopia',
    'HB': 'Burundi',
    'HC': 'Somalia (including Somaliland)',
    'HD': 'Djibouti',
    'HE': 'Egypt',
    'HH': 'Eritrea',
    'HK': 'Kenya',
    'HL': 'Libya',
    'HR': 'Rwanda',
    'HS': 'Sudan and South Sudan',
    'HT': 'Tanzania',
    'HU': 'Uganda',
    'K0': 'Contiguous United States',
    'K1': 'Contiguous United States',
    'K2': 'Contiguous United States',
    'K3': 'Contiguous United States',
    'K4': 'Contiguous United States',
    'K5': 'Contiguous United States',
    'K6': 'Contiguous United States',
    'K7': 'Contiguous United States',
    'K8': 'Contiguous United States',
    'K9': 'Contiguous United States',
    'KA': 'Contiguous United States',
    'KB': 'Contiguous United States',
    'KC': 'Contiguous United States',
    'KD': 'Contiguous United States',
    'KE': 'Contiguous United States',
    'KF': 'Contiguous United States',
    'KG': 'Contiguous United States',
    'KH': 'Contiguous United States',
    'KI': 'Contiguous United States',
    'KJ': 'Contiguous United States',
    'KK': 'Contiguous United States',
    'KL': 'Contiguous United States',
    'KM': 'Contiguous United States',
    'KN': 'Contiguous United States',
    'KO': 'Contiguous United States',
    'KP': 'Contiguous United States',
    'KR': 'Contiguous United States',
    'KS': 'Contiguous United States',
    'KT': 'Contiguous United States',
    'KU': 'Contiguous United States',
    'KV': 'Contiguous United States',
    'KW': 'Contiguous United States',
    'KX': 'Contiguous United States',
    'KY': 'Contiguous United States',
    'KZ': 'Contiguous United States',
    'LA': 'Albania',
    'LB': 'Bulgaria',
    'LC': 'Cyprus',
    'LD': 'Croatia',
    'LE': 'Spain (mainland section and Balearic Islands)',
    'LF': 'France (Metropolitan France; including Saint-Pierre and Miquelon)',
    'LG': 'Greece',
    'LH': 'Hungary',
    'LI': 'Italy',
    'LJ': 'Slovenia',
    'LK': 'Czech Republic',
    'LL': 'Israel',
    'LM': 'Malta',
    'LN': 'Monaco',
    'LO': 'Austria',
    'LP': 'Portugal (including the Azores and Madeira)',
    'LQ': 'Bosnia and Herzegovina',
    'LR': 'Romania',
    'LS': 'Switzerland',
    'LT': 'Turkey',
    'LU': 'Moldova',
    'LV': 'Palestine/Palestinian territories',
    'LW': 'North Macedonia',
    'LX': 'Gibraltar',
    'LY': 'Serbia and Montenegro',
    'LZ': 'Slovakia',
    'MB': 'Turks and Caicos Islands',
    'MD': 'Dominican Republic',
    'MG': 'Guatemala',
    'MH': 'Honduras',
    'MK': 'Jamaica',
    'MM': 'Mexico',
    'MN': 'Nicaragua',
    'MP': 'Panama',
    'MR': 'Costa Rica',
    'MS': 'El Salvador',
    'MT': 'Haiti',
    'MU': 'Cuba',
    'MW': 'Cayman Islands',
    'MY': 'Bahamas',
    'MZ': 'Belize',
    'NC': 'Cook Islands',
    'NF': 'Fiji, Tonga',
    'NG': 'Kiribati (Gilbert Islands), Tuvalu',
    'NI': 'Niue',
    'NL': 'France (Wallis and Futuna)',
    'NS': 'Samoa, United States (American Samoa)',
    'NT': 'France (French Polynesia)',
    'NV': 'Vanuatu',
    'NW': 'France (New Caledonia)',
    'NZ': 'New Zealand, Antarctica',
    'OA': 'Afghanistan',
    'OB': 'Bahrain',
    'OE': 'Saudi Arabia',
    'OI': 'Iran',
    'OJ': 'Jordan and the West Bank',
    'OK': 'Kuwait',
    'OL': 'Lebanon',
    'OM': 'United Arab Emirates',
    'OO': 'Oman',
    'OP': 'Pakistan',
    'OR': 'Iraq',
    'OS': 'Syria',
    'OT': 'Qatar',
    'OY': 'Yemen',
    'PA': 'US (Alaska)',
    'PB': 'US (Baker Island)',
    'PC': 'Kiribati (Canton Airfield, Phoenix Islands)',
    'PF': 'US (Alaska)',
    'PG': 'US (Guam, Northern Mariana Islands)',
    'PH': 'US (Hawaii)',
    'PJ': 'US (Johnston Atoll)',
    'PK': 'Marshall Islands',
    'PL': 'Kiribati (Line Islands)',
    'PM': 'US (Midway Island)',
    'PO': 'US (Alaska)',
    'PP': 'US (Alaska)',
    'PT': 'Federated States of Micronesia, Palau',
    'PW': 'US (Wake Island)',
    'RC': 'Republic of China (Taiwan)',
    'RJ': 'Japan (Mainland)',
    'RK': 'Republic of Korea (South Korea)',
    'RO': 'Japan (Okinawa)',
    'RP': 'Philippines',
    'S1': 'Argentina',
    'SA': 'Argentina',
    'SB': 'Brazil',
    'SC': 'Chile (including Easter Island)',
    'SD': 'Brazil',
    'SE': 'Ecuador',
    'SF': 'United Kingdom (Falkland Islands)',
    'SG': 'Paraguay',
    'SH': 'Chile',
    'SI': 'Brazil',
    'SJ': 'Brazil',
    'SK': 'Colombia',
    'SL': 'Bolivia',
    'SM': 'Suriname',
    'SN': 'Brazil',
    'SO': 'France (French Guiana)',
    'SP': 'Peru',
    'SS': 'Brazil',
    'SU': 'Uruguay',
    'SV': 'Venezuela',
    'SW': 'Brazil',
    'SY': 'Guyana',
    'TA': 'Antigua and Barbuda',
    'TB': 'Barbados',
    'TD': 'Dominica',
    'TF': 'France (Guadeloupe, Martinique, Saint Barthélemy, Saint Martin)',
    'TG': 'Grenada',
    'TI': 'US (U.S. Virgin Islands)',
    'TJ': 'US (Puerto Rico)',
    'TK': 'Saint Kitts and Nevis',
    'TL': 'Saint Lucia',
    'TN': 'Caribbean Netherlands, Aruba, Curaçao, Sint Maarten',
    'TQ': 'UK (Anguilla)',
    'TR': 'UK (Montserrat)',
    'TT': 'Trinidad and Tobago',
    'TU': 'UK (British Virgin Islands)',
    'TV': 'Saint Vincent and the Grenadines',
    'TX': 'UK (Bermuda)',
    'U0': 'Russia',
    'U1': 'Russia',
    'U3': 'Russia',
    'U4': 'Russia',
    'U5': 'Russia',
    'U6': 'Russia',
    'U7': 'Russia',
    'U8': 'Russia',
    'U9': 'Russia',
    'UA': 'Kazakhstan',
    'UB': 'Azerbaijan',
    'UC': 'Kyrgyzstan',
    'UD': 'Armenia',
    'UE': 'Russia',
    'UF': 'Russia',
    'UG': 'Georgia',
    'UH': 'Russia',
    'UI': 'Russia',
    'UK': 'Ukraine',
    'UL': 'Russia',
    'UM': 'Belarus and Russia (Kaliningrad Oblast)',
    'UN': 'Russia',
    'UO': 'Russia',
    'UR': 'Russia',
    'US': 'Russia',
    'UT': 'Tajikistan, Turkmenistan, Uzbekistan',
    'UU': 'Russia',
    'UW': 'Russia',
    'VA': 'West India',
    'VC': 'Sri Lanka',
    'VD': 'Cambodia',
    'VE': 'East India',
    'VG': 'Bangladesh',
    'VH': 'Hong Kong',
    'VI': 'North India',
    'VL': 'Laos',
    'VM': 'Macau',
    'VN': 'Nepal',
    'VO': 'South India',
    'VQ': 'Bhutan',
    'VR': 'Maldives',
    'VT': 'Thailand',
    'VV': 'Vietnam',
    'VY': 'Myanmar',
    'WA': 'Indonesia',
    'WB': 'Brunei, Malaysia (East Malaysia)',
    'WI': 'Indonesia',
    'WM': 'Malaysia (Peninsular Malaysia)',
    'WP': 'Timor-Leste',
    'WQ': 'Indonesia',
    'WR': 'Indonesia',
    'WS': 'Singapore',
    'Y0': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y1': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y2': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y3': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y4': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y5': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y6': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y7': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y8': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Y9': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YA': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YB': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YC': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YD': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YE': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YF': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YG': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YH': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YI': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YJ': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YK': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YL': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YM': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YN': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YO': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YP': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YQ': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YR': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YS': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YT': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YU': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YV': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YW': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'YY': 'Australia (including Norfolk Island, Christmas Island and Cocos (Keeling) Islands)',
    'Z0': 'Mainland China)',
    'Z1': 'Mainland China)',
    'Z2': 'Mainland China)',
    'Z4': 'Mainland China)',
    'Z5': 'Mainland China)',
    'Z8': 'Mainland China)',
    'Z9': 'Mainland China)',
    'ZB': 'Mainland China)',
    'ZG': 'Mainland China)',
    'ZH': 'Mainland China)',
    'ZJ': 'Mainland China)',
    'ZK': 'North Korea',
    'ZL': 'Mainland China)',
    'ZM': 'Mongolia',
    'ZP': 'Mainland China)',
    'ZS': 'Mainland China)',
    'ZU': 'Mainland China)',
    'ZW': 'Mainland China)',
    'ZY': 'Mainland China)',
    'ZZ': 'Unknown Country',
}
